#include "Adafruit_FreeTouch.h"

#define PIN_LED_R 15
#define PIN_LED_G 16
#define PIN_LED_B 17

#define PIN_BUZZER 2

#define N_PAD 5

#define PAD_THRESH 600

//#define DEBUG

const int pins_piano[N_PAD] = {3, 4, 5, 6, 7};

const byte col_piano[N_PAD][3] = {{255,   0,   0},
                                  {127, 127,   0},
                                  {  0, 255,   0},
                                  {  0, 127, 127},
                                  {  0,   0, 255}};

const int tone_piano[N_PAD] = {440, 494, 392, 196, 294};

Adafruit_FreeTouch qt_array[N_PAD];

bool error = false;

int i_trig = -1;

void setup() {
  SerialUSB.begin(0);

  // RGB LED
  digitalWrite(PIN_LED_R, HIGH);
  digitalWrite(PIN_LED_G, HIGH);
  digitalWrite(PIN_LED_B, HIGH);
  pinMode(PIN_LED_R, OUTPUT);
  pinMode(PIN_LED_G, OUTPUT);
  pinMode(PIN_LED_B, OUTPUT);

  // BUZZER
  pinMode(PIN_BUZZER, OUTPUT);

  // CAPACITIVE TOUCH
  for (int i = 0; i < N_PAD; i++) {
    qt_array[i] = Adafruit_FreeTouch(pins_piano[i], OVERSAMPLE_4, RESISTOR_50K, FREQ_MODE_NONE);
    if (!qt_array[i].begin()) {
        SerialUSB.print("Failed to begin qt on pin ");
        SerialUSB.println(pins_piano[i]);
        error = true;
    }
  }
}


void show_col(const byte* rgb) {
  // blue LED is about half the brightness, compensate here
  analogWrite(PIN_LED_R, 255-rgb[0]/2);
  analogWrite(PIN_LED_G, 255-rgb[1]/2);
  analogWrite(PIN_LED_B, 255-rgb[2]);
}


void loop() {
  if (error) return;

  #ifdef DEBUG
  for (int i = 0; i < N_PAD; i++) {
    int val = qt_array[i].measure();
    SerialUSB.print(val);
    SerialUSB.print(" ");
  }
  SerialUSB.println("");

  delay(100);
  return;
  #endif

  int i_trig_new = -1;
  
  for (int i = 0; i < N_PAD; i++) {
    int val = qt_array[i].measure();

    if (val > PAD_THRESH) {
      i_trig_new = i;
      break;
    }
  }

  if (i_trig_new != i_trig) {
    // update

    i_trig = i_trig_new;

    if (i_trig != -1) {
      // new tone
      show_col(col_piano[i_trig]);
      tone(PIN_BUZZER, tone_piano[i_trig]);
    } else {
      // nothing pressed
      const byte col_off[3] = {0, 0, 0};
      show_col(col_off);
      noTone(PIN_BUZZER);
    }
  }

  delay(2);
}
